﻿Imports System.Drawing.Drawing2D

Public Class Form1
    ' The points.
    Private Points() As PointF

    ' The solutions.
    Private Solutions As List(Of Integer())

    ' The solution we should display.
    Private CurrentSolution As Integer = 100

    ' Define the points and solutions.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        DoubleBuffered = True

        ' Define the points.
        Dim dy As Single = CSng(ClientSize.Height / 4)
        Dim dx As Single = CSng(dy / Math.Sqrt(3))
        Dim top_x As Single = CSng(ClientSize.Width / 2)
        Dim top_y As Single = -dy / 2
        Points = New PointF() _
        { _
            New PointF(top_x - dx, top_y + dy), _
            New PointF(top_x + dx, top_y + dy), _
            New PointF(top_x - 2 * dx, top_y + 2 * dy), _
            New PointF(top_x - 0 * dx, top_y + 2 * dy), _
            New PointF(top_x + 2 * dx, top_y + 2 * dy), _
            New PointF(top_x - 3 * dx, top_y + 3 * dy), _
            New PointF(top_x - 1 * dx, top_y + 3 * dy), _
            New PointF(top_x + 1 * dx, top_y + 3 * dy), _
            New PointF(top_x + 3 * dx, top_y + 3 * dy), _
            New PointF(top_x - 2 * dx, top_y + 4 * dy), _
            New PointF(top_x - 0 * dx, top_y + 4 * dy), _
            New PointF(top_x + 2 * dx, top_y + 4 * dy) _
        }

        ' Define the solutions.
        Solutions = New List(Of Integer())()

        ' E-mail from Rod Stephens 11/28/11 re: statement above:
        '
        ' This allocates a list that can contain arrays of integers.
        '
        ' The final parentheses are optional. They're the (empty) parameter list passed to the constructor
        ' for the object being created by New. I try to remember [to] include them for consistency, although in
        ' Visual Basic you can leave them out if they are empty.

        ' Insert your code here...

        ' Todd Sabuncu
        ' amsabuncu@gmail.com
        ' November 30, 2011

        Dim v1, v2, v3 As PointF
        Dim e1, e2, e3 As Single
        Dim sortedVs() As Integer
        Dim ti, tj, tk As Integer
        Dim solutionExists As Boolean

        ' Start with first vertex (For ii loop) and look at other vertices (For jj and For kk loops).
        ' Note: Solutions is zero-indexed.
        For ii As Integer = 0 To (Points.Count - 1) Step 1
            v1 = Points(ii)
            For jj As Integer = 0 To (Points.Count - 1) Step 1
                If jj = ii Then
                    ' This vertex is the same as v1, so skip.
                    Continue For
                End If
                v2 = Points(jj)
                ' Get length of edge from v1 to v2.
                GetEdgeLength(v1, v2, e1)
                For kk As Integer = 0 To (Points.Count - 1) Step 1
                    If kk = ii Or kk = jj Then
                        ' This vertex is the same as either v1 or v2, so skip.
                        Continue For
                    End If
                    v3 = Points(kk)
                    ' Get length of edge from v2 to v3.
                    GetEdgeLength(v2, v3, e2)
                    If (e1 <> e2) Then
                        ' First two edges have different lengths; not equilateral.
                        ' Look no further.
                        Continue For
                    Else
                        ' Get length of edge from v3 back to v1.
                        GetEdgeLength(v3, v1, e3)
                        If e3 = e2 Then ' or if e3 = e1, since e1 and e2 would have to be identical at this point.
                            ' Equilateral!  Sort the vertices to avoid identical cases such as 0,2,3 and 0,3,2
                            ' that only differ in ordering of the vertices.
                            sortedVs = New Integer() {ii, jj, kk}
                            Array.Sort(sortedVs)
                            ti = sortedVs(0)
                            tj = sortedVs(1)
                            tk = sortedVs(2)
                            ' Go through Solutions list to see whether we have already inserted this triangle.
                            solutionExists = False
                            If Solutions.Count > 0 Then
                                For mm As Integer = 0 To (Solutions.Count - 1)
                                    If ((ti = Solutions(mm)(0)) And _
                                        (tj = Solutions(mm)(1)) And _
                                        (tk = Solutions(mm)(2))) Then
                                        solutionExists = True ' Triangle with same vertices already in Solutions list.
                                        Exit For
                                    End If
                                Next
                            End If
                            If solutionExists = False Then
                                Solutions.Add(sortedVs)
                            End If
                        End If
                    End If
                Next
            Next
        Next
        Debug.Write("# of unique solutions: ")
        Debug.WriteLine(Solutions.Count)
    End Sub

    ' Calculate length of line between any two PointF's.
    Private Sub GetEdgeLength(ByVal startVert As PointF, _
                              ByVal endVert As PointF, _
                              ByRef EdgeLength As Single)

        Dim distX, distY As Single

        If startVert.X = endVert.X Then
            ' Straight line down Y axis; difference of Y coordinates gives us line length.
            EdgeLength = CSng(Math.Round(Math.Abs(endVert.Y - startVert.Y), 1))
        ElseIf startVert.Y = endVert.Y Then
            ' Straight line down X axis; difference of X coordinates gives us line length.
            EdgeLength = CSng(Math.Round(Math.Abs(endVert.X - startVert.X), 1))
        Else
            ' Slope; use Pythagorean theorem.
            distX = Math.Abs(endVert.X - startVert.X)
            distY = Math.Abs(endVert.Y - startVert.Y)
            ' We round length to 1 digit after decimal place.  Works for client area of height 264, width 334.
            EdgeLength = CSng(Math.Round(Math.Sqrt((distX * distX) + (distY * distY)), 1))
        End If

    End Sub

    ' Draw the circles and any triangles currently defined.
    Private Sub Form1_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint
        e.Graphics.SmoothingMode = SmoothingMode.AntiAlias

        ' Draw the points.
        Const radius As Single = 5
        For Each pt As PointF In Points
            e.Graphics.FillEllipse(Brushes.Blue, _
                pt.X - radius, pt.Y - radius, _
                2 * radius, 2 * radius)
        Next pt

        ' Draw the current solution.
        If (CurrentSolution < 0) Then
            ' Draw all solutions.
            For i As Integer = 0 To Solutions.Count - 1
                DrawSolution(e.Graphics, i)
            Next i
        Else
            ' Draw the current solution.
            DrawSolution(e.Graphics, CurrentSolution)
        End If
    End Sub

    ' Draw a solution.
    Private Sub DrawSolution(ByVal gr As Graphics, ByVal solution_num As Integer)
        If (solution_num >= Solutions.Count) Then Return

        ' Make a list of the points in this solution.
        Dim pts As New List(Of PointF)()
        For Each i As Integer In Solutions(solution_num)
            pts.Add(Points(i))
        Next i
        gr.DrawPolygon(Pens.Red, pts.ToArray())
    End Sub

    ' Start showing solutions.
    Private Sub btnShowSolutions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnShowSolutions.Click
        ' Disable the button.
        btnShowSolutions.Enabled = False

        ' Start at the first solution.
        CurrentSolution = 0

        ' Start the timer.
        tmrChangeSolution.Enabled = True

        ' Redraw.
        Refresh()
    End Sub

    ' Show the next solution.
    Private Sub tmrChangeSolution_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tmrChangeSolution.Tick
        ' Increment CurrentSolution. If the result is greater than the
        ' last solution's index, disable the timer.
        CurrentSolution += 1
        tmrChangeSolution.Enabled = (CurrentSolution < Solutions.Count)

        ' If we're done drawing, enable the button.
        btnShowSolutions.Enabled = (Not tmrChangeSolution.Enabled)

        ' Redraw.
        Refresh()
    End Sub

    ' Show all of the solutions.
    Private Sub btnShowAllSolutions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnShowAllSolutions.Click
        CurrentSolution = -1
        tmrChangeSolution.Enabled = False
        btnShowSolutions.Enabled = True
        Refresh()
    End Sub
End Class
