﻿Imports System.Drawing.Drawing2D

Public Class Form1
    ' The points.
    Private Points() As PointF

    ' The solutions.
    Private Solutions As List(Of Integer())

    ' The solution we should display.
    Private CurrentSolution As Integer = 100

    ' Define the points and solutions.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        DoubleBuffered = True

        ' Define the points.
        Dim dy As Single = CSng(ClientSize.Height / 4)
        Dim dx As Single = CSng(dy / Math.Sqrt(3))
        Dim top_x As Single = CSng(ClientSize.Width / 2)
        Dim top_y As Single = -dy / 2
        Points = New PointF() _
        { _
            New PointF(top_x - dx, top_y + dy), _
            New PointF(top_x + dx, top_y + dy), _
            New PointF(top_x - 2 * dx, top_y + 2 * dy), _
            New PointF(top_x - 0 * dx, top_y + 2 * dy), _
            New PointF(top_x + 2 * dx, top_y + 2 * dy), _
            New PointF(top_x - 3 * dx, top_y + 3 * dy), _
            New PointF(top_x - 1 * dx, top_y + 3 * dy), _
            New PointF(top_x + 1 * dx, top_y + 3 * dy), _
            New PointF(top_x + 3 * dx, top_y + 3 * dy), _
            New PointF(top_x - 2 * dx, top_y + 4 * dy), _
            New PointF(top_x - 0 * dx, top_y + 4 * dy), _
            New PointF(top_x + 2 * dx, top_y + 4 * dy) _
        }

        ' Define the solutions.
        Solutions = New List(Of Integer())()

        'Insert your code here...
        For i As Integer = 0 To Points.Length - 1
            For j As Integer = 0 To Points.Length - 1
                If j = i Then Continue For
                For k As Integer = 0 To Points.Length - 1
                    If k = i OrElse k = j Then Continue For
                    Dim p1 As PointF = Points(i)
                    Dim p2 As PointF = Points(j)
                    Dim p3 As PointF = Points(k)
                    If DistanceSquared(p1, p2) = DistanceSquared(p2, p3) AndAlso _
                       DistanceSquared(p1, p2) = DistanceSquared(p1, p3) Then
                        Dim triangle As Integer() = {i, j, k}
                        Array.Sort(triangle)
                        Dim skip As Boolean = False
                        For Each solution In Solutions
                            If solution(0) = triangle(0) _
                            AndAlso solution(1) = triangle(1) _
                            AndAlso solution(2) = triangle(2) Then
                                skip = True
                            End If
                        Next
                        If Not skip Then
                            Solutions.Add(triangle)
                            Console.WriteLine(String.Format("solution {0}: {1}, {2}, {3}", Solutions.Count, i, j, k))
                        End If
                    End If
                Next
            Next
        Next
    End Sub

    ' Calculate length of line between any two PointF's.
    Private Sub GetEdgeLength(ByVal startVert As PointF, _
                              ByVal endVert As PointF, _
                              ByRef EdgeLength As Single)

        Dim distX, distY As Single

        If startVert.X = endVert.X Then
            ' Straight line down Y axis; difference of Y coordinates gives us line length.
            EdgeLength = CSng(Math.Round(Math.Abs(endVert.Y - startVert.Y), 1))
        ElseIf startVert.Y = endVert.Y Then
            ' Straight line down X axis; difference of X coordinates gives us line length.
            EdgeLength = CSng(Math.Round(Math.Abs(endVert.X - startVert.X), 1))
        Else
            ' Slope; use Pythagorean theorem.
            distX = Math.Abs(endVert.X - startVert.X)
            distY = Math.Abs(endVert.Y - startVert.Y)
            ' We round length to 1 digit after decimal place.  Works for client area of height 264, width 334.
            EdgeLength = CSng(Math.Round(Math.Sqrt((distX * distX) + (distY * distY)), 1))
        End If

    End Sub

    ' Draw the circles and any triangles currently defined.
    Private Sub Form1_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint
        e.Graphics.SmoothingMode = SmoothingMode.AntiAlias

        ' Draw the points.
        Const radius As Single = 5
        For Each pt As PointF In Points
            e.Graphics.FillEllipse(Brushes.Blue, _
                pt.X - radius, pt.Y - radius, _
                2 * radius, 2 * radius)
        Next pt

        ' Draw the current solution.
        If (CurrentSolution < 0) Then
            ' Draw all solutions.
            For i As Integer = 0 To Solutions.Count - 1
                DrawSolution(e.Graphics, i)
            Next i
        Else
            ' Draw the current solution.
            DrawSolution(e.Graphics, CurrentSolution)
        End If
    End Sub

    ' Draw a solution.
    Private Sub DrawSolution(ByVal gr As Graphics, ByVal solution_num As Integer)
        If (solution_num >= Solutions.Count) Then Return

        ' Make a list of the points in this solution.
        Dim pts As New List(Of PointF)()
        For Each i As Integer In Solutions(solution_num)
            pts.Add(Points(i))
        Next i
        gr.DrawPolygon(Pens.Red, pts.ToArray())
    End Sub

    ' Start showing solutions.
    Private Sub btnShowSolutions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnShowSolutions.Click
        ' Disable the button.
        btnShowSolutions.Enabled = False

        ' Start at the first solution.
        CurrentSolution = 0

        ' Start the timer.
        tmrChangeSolution.Enabled = True

        ' Redraw.
        Refresh()
    End Sub

    ' Show the next solution.
    Private Sub tmrChangeSolution_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tmrChangeSolution.Tick
        ' Increment CurrentSolution. If the result is greater than the
        ' last solution's index, disable the timer.
        CurrentSolution += 1
        tmrChangeSolution.Enabled = (CurrentSolution < Solutions.Count)

        ' If we're done drawing, enable the button.
        btnShowSolutions.Enabled = (Not tmrChangeSolution.Enabled)

        ' Redraw.
        Refresh()
    End Sub

    ' Show all of the solutions.
    Private Sub btnShowAllSolutions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnShowAllSolutions.Click
        CurrentSolution = -1
        tmrChangeSolution.Enabled = False
        btnShowSolutions.Enabled = True
        Refresh()
    End Sub

    Private Function DistanceSquared(ByVal p1 As PointF, ByVal p2 As PointF) As Integer
        Return CInt((p1.X - p2.X) ^ 2 + (p1.Y - p2.Y) ^ 2)
    End Function
End Class
